@@ -1,8 +1,13 @@
 package Tie::Func;
 
+use strict;
+use vars qw($VERSION);
+
+$VERSION = '1.00';
+
 sub TIEHASH {
     my ($class, $fetch, $store, $del, %const) = @_;
-    foreach (qw($fetch $store $each $del)) {
+    foreach (qw($fetch $store $del)) {
         eval "$_ = &$_ if ($_ && ref($_) !~ /CODE/);";
     }
     bless {'data' => \%const, '__fetch' => $fetch, '__store' => $store,
@@ -38,7 +43,7 @@ sub DELETE {
     my ($this, $key) = @_;
     my $code = $this->{'__del'};
     my $value = $this->{'data'}->{$key};
-    return undef if ($code && !&$code(this, $key, $value));
+    return undef if ($code && !&$code($this, $key, $value));
     delete $this->{'data'}->{$key};
     1;
 }
